/*==LICENSE==*

CyanWorlds.com Engine - MMOG client, server and tools
Copyright (C) 2011  Cyan Worlds, Inc.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Additional permissions under GNU GPL version 3 section 7

If you modify this Program, or any covered work, by linking or
combining it with any of RAD Game Tools Bink SDK, Autodesk 3ds Max SDK,
NVIDIA PhysX SDK, Microsoft DirectX SDK, OpenSSL library, Independent
JPEG Group JPEG library, Microsoft Windows Media SDK, or Apple QuickTime SDK
(or a modified version of those libraries),
containing parts covered by the terms of the Bink SDK EULA, 3ds Max EULA,
PhysX SDK EULA, DirectX SDK EULA, OpenSSL and SSLeay licenses, IJG
JPEG Library README, Windows Media SDK EULA, or QuickTime SDK EULA, the
licensors of this Program grant you additional
permission to convey the resulting work. Corresponding Source for a
non-source form of such a combination shall include the source code for
the parts of OpenSSL and IJG JPEG Library used as well as that of the covered
work.

You can contact Cyan Worlds, Inc. by email legal@cyan.com
 or by snail mail at:
      Cyan Worlds, Inc.
      14617 N Newport Hwy
      Mead, WA   99021

*==LICENSE==*/
#ifndef _pyGameScoreMsg_h_
#define _pyGameScoreMsg_h_

#include "pfMessage/pfGameScoreMsg.h"
#include "pyGlueHelpers.h"

class pyGameScoreMsg
{
protected:
    pfGameScoreMsg* fMsg;

    pyGameScoreMsg()
        : fMsg(nil)
    { }

    pyGameScoreMsg(pfGameScoreMsg* msg)
        : fMsg(msg)
    {
        hsRefCnt_SafeRef(msg);
    }

public:
    virtual ~pyGameScoreMsg()
    {
        hsRefCnt_SafeUnRef(fMsg);
    }

    // required functions for PyObject interoperability
    PYTHON_EXPOSE_TYPE; // so we can subclass
    PYTHON_CLASS_NEW_FRIEND(ptGameScoreMsg);
    static PyObject* CreateFinal(pfGameScoreMsg* msg); // returns the proper descendant. Note there is no New
    PYTHON_CLASS_CHECK_DEFINITION; // returns true if the PyObject is a pyGameScoreMsg object
    PYTHON_CLASS_CONVERT_FROM_DEFINITION(pyGameScoreMsg); // converts a PyObject to a pyGameScoreMsg (throws error if not correct type)

    plString    GetError() const;
    bool        IsValid() const;

    static void AddPlasmaClasses(PyObject* m);
};

class pyGameScoreListMsg : public pyGameScoreMsg
{
    pyGameScoreListMsg()
        : pyGameScoreMsg()
    { }

    pyGameScoreListMsg(pfGameScoreListMsg* msg)
        : pyGameScoreMsg(msg)
    { }

public:
    PYTHON_CLASS_NEW_FRIEND(ptGameScoreListMsg);
    static PyObject* New(pfGameScoreListMsg* msg);
    PYTHON_CLASS_CHECK_DEFINITION; // returns true if the PyObject is a pyGameScoreListMsg object
    PYTHON_CLASS_CONVERT_FROM_DEFINITION(pyGameScoreListMsg); // converts a PyObject to a pyGameScoreListMsg (throws error if not correct type)

    static void AddPlasmaClasses(PyObject* m);

    plString GetName() const;
    uint32_t GetOwnerID() const;
    size_t GetNumScores() const;
    PyObject* GetScore(size_t idx) const;
};

class pyGameScoreTransferMsg : public pyGameScoreMsg
{
    pyGameScoreTransferMsg()
        : pyGameScoreMsg()
    { }

    pyGameScoreTransferMsg(pfGameScoreTransferMsg* msg)
        : pyGameScoreMsg(msg)
    { }

public:
    PYTHON_CLASS_NEW_FRIEND(ptGameScoreTransferMsg);
    static PyObject* New(pfGameScoreTransferMsg* msg);
    PYTHON_CLASS_CHECK_DEFINITION; // returns true if the PyObject is a pyGameScoreTransferMsg object
    PYTHON_CLASS_CONVERT_FROM_DEFINITION(pyGameScoreTransferMsg); // converts a PyObject to a pyGameScoreTransferMsg (throws error if not correct type)

    static void AddPlasmaClasses(PyObject* m);

    PyObject* GetDestinationScore() const;
    PyObject* GetSourceScore() const;
};

class pyGameScoreUpdateMsg : public pyGameScoreMsg
{
    pyGameScoreUpdateMsg()
        : pyGameScoreMsg()
    { }

    pyGameScoreUpdateMsg(pfGameScoreUpdateMsg* msg)
        : pyGameScoreMsg(msg)
    { }

public:
    PYTHON_CLASS_NEW_FRIEND(ptGameScoreUpdateMsg);
    static PyObject* New(pfGameScoreUpdateMsg* msg);
    PYTHON_CLASS_CHECK_DEFINITION; // returns true if the PyObject is a pyGameScoreUpdateMsg object
    PYTHON_CLASS_CONVERT_FROM_DEFINITION(pyGameScoreUpdateMsg); // converts a PyObject to a pyGameScoreUpdateMsg (throws error if not correct type)

    static void AddPlasmaClasses(PyObject* m);

    PyObject* GetScore() const;
};

#endif // _pyGameScoreMsg_h_
